<?php
Namespace App\Controller\Admin;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Translation\TranslatorInterface;
use \Homegear\Homegear as Homegear;
use App\Utils\Utils;

class DashboardController extends Controller
{
    private $hg;
    private $translator;
    private $utils;

    public function __construct(TranslatorInterface $translator, Utils $utils)
    {
        $this->hg = new Homegear();
        $this->utils = $utils;
        $this->translator = $translator;
    }

    public function indexAction(Request $request)
    {
        if (!$this->utils->checkManagementRunning(false)) {
            // management not running
            $this->addFlash('warning',$this->translator->trans('flash.management_not_running',array(),'admin'));
        }
        $ca = $this->utils->checkCa();
        if (intval($ca) == -1) {
            // unable to check CA because management not running
            $this->addFlash('warning',$this->translator->trans('flash.unable_to_check_ca',array(),'admin'));
        }
        if (!$ca) {
            // no CA found
            $this->addFlash('warning',$this->translator->trans('flash.no_ca_found',array('%create.ca%' => $this->generateUrl('admin.maintenance.ca')),'admin'));
        }
        return $this->render('Admin/Dashboard/dashboard.html.twig',array(
            'sect' => $request->attributes->get('sect')
        ));
    }

    public function getDeviceStats() {
        $unreach = 0;
        $devices = $this->hg->listDevices(false,array("ID", "ADDRESS", "TYPE"));
        $messages = $this->hg->getServiceMessages();
        foreach($messages as $k => $v) {
            if ($v[1] == "UNREACH") {
                $device = explode(":",$v[0]);
                if (array_search($device[0],array_column($devices, 'ADDRESS')) !== false) {
                $unreach++;
                }
            }
        }
        return $this->render('Admin/Widgets/deviceStats.html.twig',array(
            'totalDevices' => count($devices),
            'unreachableDevices' => $unreach
        ));
    }

    public function getIpAddresses() {
        $tmp = shell_exec('ifconfig');
        $tmp = preg_split("/\n\n/",$tmp);
        foreach ($tmp as $k => $v) {
            if ($v == '') { continue; }
                if (strpos($v, 'flags')) {
                    $interface = substr($v, 0, strpos($v, 'flags'));
                } elseif (strpos($v, 'Link')) {
                $interface = substr($v, 0, strpos($v, 'Link'));
            }
            $interface = str_replace(':','',$interface);
            if (strpos($interface,'lo') === false && strpos($interface,'wlan') === false) {
                preg_match('/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/', $v, $ip);
                if(array_key_exists(0, $ip))
                {
                    $interfaces[$k]['name'] = $interface;
                    $interfaces[$k]['ip'] = $ip[0];
                }
            }
        }
        return $this->render('Admin/Widgets/interfaceList.html.twig',array(
            'interfaces' => array_values($interfaces)
        ));
    }

    public function getModules()
    {
        return $this->render('Admin/Widgets/installedModules.html.twig', array(
            'modules' => $this->hg->listModules()
        ));
    }

    public function getSysinfo()
    {
        $load = sys_getloadavg();
        $uptime = $this->_getUptime();
        return $this->render('Admin/Widgets/systemInformation.html.twig', array(
            'hostname'  => gethostname(),
            'load0'     => sprintf("%01.2f",$load[0]),
            'load1'     => sprintf("%01.2f",$load[1]),
            'load2'     => sprintf("%01.2f",$load[2]),
            'mem_total' => $this->_getServerMemoryUsage('total'),
            'mem_free'  => $this->_getServerMemoryUsage('free'),
            'uptime_days'   => $uptime['day'],
            'uptime_hours'  => sprintf("%'.02d",$uptime['hour']),
            'uptime_minutes'    => sprintf("%'.02d",$uptime['min']),
            'uptime_seconds'    => sprintf("%'.02d",$uptime['sec']),
            'date_weekday'      => strftime("%A"),
            'date_day'          => strftime("%d"),
            'date_month'        => strftime("%B"),
            'date_year'         => strftime("%Y"),
            'date_time'         => strftime("%H:%M:%S"),
            'disk_total'        => $this->formatBytes((double)disk_total_space("/")),
            'disk_used'         => $this->formatBytes(((double)disk_total_space("/") - (double)disk_free_space("/"))),
            'hg_version'        => str_replace('Homegear ', '', $this->hg->getVersion())
        ));
    }

    private function _getServerMemoryUsage($type){
        $fh = fopen('/proc/meminfo','r');
        $mem = 0;
        while ($line = fgets($fh)) {
            $pieces = array();
            if ($type == 'total' && preg_match('/^MemTotal:\s+(\d+)\skB$/', $line, $pieces)) {
                $mem = $pieces[1];
                break;
            }
            if ($type == 'free' && preg_match('/^MemFree:\s+(\d+)\skB$/', $line, $pieces)) {
                $mem = $pieces[1];
                break;
            }
        }
        fclose($fh);
        return $this->formatBytes((int)$mem * 1024);
    }

    private function _getUptime() {
        $str = @file_get_contents('/proc/uptime');
        $num = floatval($str);
        $uptime['sec'] = fmod($num, 60);
        $num = (int)($num / 60);
        $uptime['min'] = $num % 60;
        $num = (int)($num / 60);
        $uptime['hour'] = $num % 24;
        $num = (int)($num / 24);
        $uptime['day'] = $num;
        return $uptime;
    }

    private function formatBytes($bytes, $precision = 2) { 
        $units = array('B', 'KB', 'MB', 'GB', 'TB'); 
    
        $bytes = max($bytes, 0); 
        $pow = floor(($bytes ? log($bytes) : 0) / log(1024)); 
        $pow = min($pow, count($units) - 1); 
    
        // Uncomment one of the following alternatives
        //$bytes /= pow(1024, $pow);
        $bytes /= (1 << (10 * $pow)); 
        return round($bytes, $precision) . ' ' . $units[$pow]; 
    }
}
