<?php
Namespace App\Model\Admin;

use Symfony\Component\Translation\TranslatorInterface;
Use App\Utils\Utils;

/**
 * Provides admin functions for Homegear update management
 *
 */
class HomegearUpdate
{
    public $hg;
    private $updateSever = 'https://licensing.easy-smarthome.de/updates/';
    private $workDir = '/tmp';
    private $utils;
    private $translator;

    public function __construct(TranslatorInterface $translator, Utils $utils)
    {
        $this->hg = new \Homegear\Homegear;
        $this->utils = $utils;
        $this->translator = $translator;
    }

    /**
     * List all current updates
     * 
     * @return array
     */
    public function list()
    {
        if ($this->utils->checkManagementRunning(false)) {
            // only fetch update-list once an hour
            $last = $this->hg->getData('aptUpdate', 'last');
            if (!$last || ($last < (time()-3600))) {
                $this->hg->managementAptUpdate();
                $this->hg->setData('aptUpdate', 'last', time());
            }
            // check if an update is running
            if ($this->utils->checkUpdateRunning('_sys-update-running') === true || $this->utils->checkUpdateRunning('_os-update-running') === true) {
                $os = -1;
                $system = -1;
            } else {
                $os = $this->hg->managementSystemUpdateAvailable();
                $system = $this->hg->managementHomegearUpdateAvailable();
            }
            $deviceDefinition = $this->_checkDeviceDefinitionUpdates();
            return ['system' => $system, 'os' => $os, 'device_definition' => $deviceDefinition];
        }
        return false;
    }

    /**
     * Update packages
     * 
     * @return json
     */
    public function update($type = null)
    {
        if ($type === null) { return ['result' => 'error', 'msg' => 'Update-type missing']; }
        if ($type == 'device_definition') {
            $result = $this->_installDeviceDefinitionUpdate();
            if ($result === false) {
                return ['result' => 'error', 'msg' => 'Error getting update'];
            }
        } elseif ($type == 'sys') {
            if ($this->utils->checkManagementRunning(false)) {
                $this->hg->managementAptUpgrade(intval(1));
                $this->utils->saveToSession(['_sys-update-running' => $id]);
                return ['result' => 'success'];
            }
        } elseif ($type == 'os') {
            if ($this->utils->checkManagementRunning(false)) {
                $this->hg->managementAptUpgrade(intval(0));
                $this->utils->saveToSession(['_os-update-running' => $id]);
                return ['result' => 'success'];
            }
        }
        return ['result' => 'error', 'msg' => 'Error getting update'];
    }

    /**
     * Check for device definition updates
     * 
     * @return bool
     */
    private function _checkDeviceDefinitionUpdates()
    {
        $currentVersion = 0;
        if (file_exists(__DIR__.'/../../../public/device_definitions/default/.version')) {
            $currentVersion = trim(preg_replace('/\s+/', ' ',file_get_contents(__DIR__.'/../../../public/device_definitions/default/.version')));
        }
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $this->updateSever.'deviceDefinitions/update.json');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        $result = curl_exec($curl);
        curl_close($curl);
        if ($result != '') {
            $data = json_decode($result,true);
            if (is_array($data) && array_key_exists('version', $data)) {
                if (intval($data['version']) > intval($currentVersion)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Install new device definitions
     * 
     * @return bool
     */
    private function _installDeviceDefinitionUpdate()
    {
        $result = $this->_downloadFile($this->updateSever.'deviceDefinitions/deviceDefinitions.tar.gz','deviceDefinitions');
        if ($result === false) {
            return $result;
        }
        exec(escapeshellcmd("tar -xzvf ".$this->workDir."/deviceDefinitions.tar.gz -C /var/lib/homegear/ui/public/device_definitions"),$result);
        exec(escapeshellcmd('rm -rf '.$this->workDir.'/deviceDefinitions.tar.gz'));
        return true;
    }

    /**
     * Download a file (for ui/device definition update)
     * 
     * @return bool
     */
    private function _downloadFile($url = false, $file = false)
    {
        if ($url === false || $file === false) { return false; }
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $data = curl_exec($ch);
        $download = fopen($this->workDir."/$file.tar.gz",'w');
        fputs($download,$data);
        fclose($download);
        curl_close($ch);
        if (file_exists($this->workDir."/$file.tar.gz") && filesize($this->workDir."/$file.tar.gz") > 300) {
            return true;
        } else {
            return false;
        }
    }
}